#!/usr/bin/perl -w
#
#	Converts the Yahoo! Webscope dataset to the format described in:
# 	Karatzoglou et al.: Multiverse Recommendation: N-dimensional Tensor Factorization for Context-aware Collaborative Filtering
# 
#	Author:   Steffen Rendle, http://www.libfm.org/
#	modified: 2011-07-14
#
#	Copyright 2010-2011 Steffen Rendle, see license.txt for more information



use Getopt::Long;
use strict;

my $alpha = 0.1;
my $beta = 0.1;
GetOptions (
		"alpha=f" 	=> \$alpha,
		"beta=f" => \$beta,
	);
	

my $file_data = 'webscope.user_movie.txt';
my $file_user = 'ydata-ymovies-user-demographics-v1_0.txt';

# read gender information
my %user_age;
my %user_gender;
{
	open my $FILE_IN, '<', $file_user;
	while (<$FILE_IN>) {
		my $line = $_;
		chomp $line;
		if ($line eq "") {next;}
		my ($ext_user_id, $ext_year, $ext_gender) = split(/\s/,$line);
		$ext_user_id += 0;
		
		my $age_id = 0;
		if ($ext_year ne "undef") {
			$ext_year += 0;
			my $age = 2010-$ext_year;
			if ($age < 18) {
				$age_id = 1;
			} elsif ($age <= 50) {
				$age_id = 2;
			} else {
				$age_id = 3;
			}
		}
		$user_age{$ext_user_id} = $age_id;
  		
		my $gender_id;
		if ($ext_gender eq "m") {
			$gender_id = 0;
		} else {
			$gender_id = 1;
		}
		$user_gender{$ext_user_id} =  $gender_id;

	}
	close $FILE_IN;
}


# read and write data
{
	my %item_id;
	my $item_id_cntr = 0;
	my %item_is_picked_for_altering;
	open my $FILE_IN, '<', $file_data;
	while (<$FILE_IN>) {
		my $line = $_;
		chomp $line;
		if ($line eq "") {next;}
		my ($ext_user_id, $ext_item_id, $rating_skip, $rating) = split(/\s/,$line);
		$ext_user_id += 0;
		$ext_item_id += 0;
		$rating += 0;
		if (! exists $item_id{$ext_item_id}) {
			$item_id{$ext_item_id} = $item_id_cntr;
			# "randomly choose α ∗ 100% items from the dataset" 
			$item_is_picked_for_altering{$ext_item_id} = (rand() < $alpha);
			$item_id_cntr++;
		}
	

		# "replace the gender feature with a new artificial feature c ∈ {0, 1}
		#  that was assigned randomly to the value 1 or 0 for each rating." 
		my $artifical_context = (rand() < 0.5);		
		
		# "choose α ∗ 100% items from the dataset, and for these items [...] randomly
		#  pick β ∗ 100% of the ratings to modify. [...] increase (or de-
		#  crease) the rating value by one if c = 1 (c = 0) if the rating
		#  value was not already 5 (1)."
		if ($item_is_picked_for_altering{$ext_item_id}) {
			if ($artifical_context) {
				if ($rating <= 4) { $rating +=1; }
			} else {
				if ($rating >= 2) { $rating -=1; }
			}
		}
		my @out_fields = ($ext_user_id, $item_id{$ext_item_id}, $user_age{$ext_user_id}, $artifical_context+0, $rating);
		print join("\t", @out_fields), "\n"; 
	}
	close $FILE_IN;
}

